import java.util.*;
import no.hiof.imagepr.*;
import java.io.*;

public class FinalProject {

	public static void main(String[] args){
		new FinalProject(args);
	}

	private RGBImage img;			// the image we are finding key points on.
	private double[][] averageArea; // the array that stores the area values for each pixel

	private double circleRadius;	// the "big" circle radius
	private double areaRadius;		// the "little" circle radius
	private int pointCount;			// # of gradient directions to be computed.
	private double gradThresh;		// smallest gradient magnitude to continue a chain

	private int closestCount;		// # of points to look at when looking at geometry
									// of neighboring key points.

	public FinalProject(String[] args){

		pointCount = Integer.parseInt(args[0]);

		StringTokenizer st = new StringTokenizer(args[1],",");
		// what size "big" circle radius to start with.
		double startRadius = Double.parseDouble(st.nextToken());
		// what size "big" cirlce radius to end with.
		double endRadius = Double.parseDouble(st.nextToken());
		// how much to increment the circle size by
		double incRadius = Double.parseDouble(st.nextToken());

		// # of key points to detect.
		int keyPointCount = Integer.parseInt(args[2]);

		st = new StringTokenizer(args[3],",");

		// radius to search for points in the next image
		int searchRadius = Integer.parseInt(args[4]);
		// how many points should we track
		int trackedPointsCount = Integer.parseInt(args[5]);

		closestCount = Integer.parseInt(args[6]);

		// initialize an array that stores the key points found in each input image
		PixelInfo[][] keyPoints = new PixelInfo[st.countTokens()][];
		int i = 0;
		while(st.hasMoreTokens()){
			keyPoints[i] = getKeyPointGrid(st.nextToken(),startRadius,endRadius,incRadius,keyPointCount);
			i++;
		}

		for(i=1;i<keyPoints.length;i++){

			// the key points in the image before this image
			PixelInfo[] keyPoints1 = keyPoints[i-1];
			// the key points in this image
			PixelInfo[] keyPoints2 = keyPoints[i];

			DoubleObjectPair[] bestPairs = new DoubleObjectPair[trackedPointsCount];
			int counter = 0;
			boolean mustSort = false;

			// for each key point in the first image
			for(int j=0;j<keyPoints1.length;j++){

				DoubleObjectPair bestPair = null;
				double bestDistance = Double.POSITIVE_INFINITY;

				// for each key point in the second image
				for(int k=0;k<keyPoints2.length;k++){
					double distance = distance(keyPoints1[j].row,keyPoints2[k].row,keyPoints1[j].col,keyPoints2[k].col);
					if(distance <= searchRadius){

						// determine how well these two points match
						double diff = getDifference(keyPoints1[j],keyPoints1,keyPoints2[k],keyPoints2);

						DoubleObjectPair pair = new DoubleObjectPair(diff,keyPoints1[j].col+" "+keyPoints1[j].row+" "+keyPoints2[k].col+" "+keyPoints2[k].row);
						// if I haven't found a match OR I found a better match OR I found an equal match that is closer
						if(bestPair==null || pair.compareTo(bestPair) < 0 || (pair.compareTo(bestPair)==0 && bestDistance > distance)){
							bestPair = pair;
							bestDistance = distance;
						}

					}
				}

				// if a pair was found
				if(bestPair != null){
					if(counter < bestPairs.length){
						bestPairs[counter] = bestPair;
						mustSort = true;
					}else{
						if(mustSort){
							Arrays.sort(bestPairs);
							mustSort = false;
						}
						if(bestPair.compareTo(bestPairs[bestPairs.length-1]) < 0){
							bestPairs[bestPairs.length-1] = bestPair;
							mustSort = true;
						}
					}
					counter++;
				}
			}

			if(counter >= bestPairs.length && mustSort){
				Arrays.sort(bestPairs);
			}

			// output the tracked points in a format that can be read into matlab
			// to mosaic the images.
			System.out.println("T_"+(i-1)+"_"+i+" = [");
			for(int b=0;b<bestPairs.length;b++){
				if(bestPairs[b]!=null){
					System.out.println(bestPairs[b].object + " 0;");
				}
			}
			System.out.println();

		}

	}

	// determine how well the two key points p1 and p2 match
	public double getDifference(PixelInfo p1,PixelInfo[] keyPoints1,PixelInfo p2,PixelInfo[] keyPoints2){

		double minDiff = Double.POSITIVE_INFINITY;
		int d1,d2;
		double v1,v2,totalDiff,dist;

		// find the points closest to p1 in keyPoints1
		DoubleObjectPair[] closest1 = new DoubleObjectPair[closestCount];
		int counter = 0;
		boolean mustSort = true;
		for(int k=0;k<keyPoints1.length;k++){
			dist = distance(p1.row,keyPoints1[k].row,p1.col,keyPoints1[k].col);
			DoubleObjectPair pair = new DoubleObjectPair(dist, new Point(keyPoints1[k].col-p1.col, -(keyPoints1[k].row-p1.row)) );
			if(counter < closest1.length){
				closest1[counter] = pair;
				mustSort = true;
			}else{
				if(mustSort){
					Arrays.sort(closest1);
					mustSort = false;
				}
				if(pair.compareTo(closest1[closest1.length-1]) < 0){
					closest1[closest1.length-1] = pair;
					mustSort = true;
				}
			}
			counter++;
		}
		if(mustSort){
			Arrays.sort(closest1);
		}

		// find the points closest to p2 in keyPoints2
		DoubleObjectPair[] closest2 = new DoubleObjectPair[closestCount];
		counter = 0;
		mustSort = true;
		for(int k=0;k<keyPoints2.length;k++){
			dist = distance(p2.row,keyPoints2[k].row,p2.col,keyPoints2[k].col);
			DoubleObjectPair pair = new DoubleObjectPair(dist, new Point(keyPoints2[k].col-p2.col, -(keyPoints2[k].row-p2.row)) );
			if(counter < closest2.length){
				closest2[counter] = pair;
				mustSort = true;
			}else{
				if(mustSort){
					Arrays.sort(closest2);
					mustSort = false;
				}
				if(pair.compareTo(closest2[closest2.length-1]) < 0){
					closest2[closest2.length-1] = pair;
					mustSort = true;
				}
			}
			counter++;
		}
		if(mustSort){
			Arrays.sort(closest2);
		}

		// for each max direction of this pixel
		for(int i=0;i<p1.maxDirections.size();i++){
			d1 = ((Integer)p1.maxDirections.elementAt(i)).intValue();
			// for each max direction of the other pixel
			for(int j=0;j<p2.maxDirections.size();j++){
				d2 = ((Integer)p2.maxDirections.elementAt(j)).intValue();

				// determine theta for the major direction of d1
				double m1 = (d1/(double)p1.circleValues.length)*2*Math.PI;
				// determine theta for the major direction of d2
				double m2 = (d2/(double)p2.circleValues.length)*2*Math.PI;

				// rotate the points in closest1 by m1 and convert to polar
				double averageDist = 0;
				for(int p=0;p<closest1.length;p++){
					Point pt = (Point)(closest1[p].object);
					// convert to polar cordinates
					double[] polar = convertToPolar(pt.x,pt.y);
					// rotate the point by m1
					polar[1] = (polar[1]+m1)%(2*Math.PI);
					averageDist += polar[0];
					// store the r and theta to convert back to cartesian in the next loop
					pt.x = polar[0];
					pt.y = polar[1];
				}
				averageDist /= closest1.length;
				// normalize their distance from p1 and convert back to cartesian
				for(int p=0;p<closest1.length;p++){
					Point pt = (Point)(closest1[p].object);
					// normalize the radius in polar coordinates
					pt.x = pt.x/averageDist;
					// convert back to cartesian
					double[] cartesian = convertToCartesian(pt.x,pt.y);
					pt.x = cartesian[0];
					pt.y = cartesian[1];
				}

				// rotate the points in closest2 by m2 and convert to polar
				averageDist = 0;
				for(int p=0;p<closest2.length;p++){
					Point pt = (Point)(closest2[p].object);
					// convert to polar cordinates
					double[] polar = convertToPolar(pt.x,pt.y);
					// rotate the point by m2
					polar[1] = (polar[1]+m2)%(2*Math.PI);
					averageDist += polar[0];
					// store the r and theta to convert back to cartesian in the next loop
					pt.x = polar[0];
					pt.y = polar[1];
				}
				averageDist /= closest2.length;
				// normalize their distance from p2 and convert back to cartesian
				for(int p=0;p<closest2.length;p++){
					Point pt = (Point)(closest2[p].object);
					// normalize the radius in polar coordinates
					pt.x = pt.x/averageDist;
					// convert back to cartesian
					double[] cartesian = convertToCartesian(pt.x,pt.y);
					pt.x = cartesian[0];
					pt.y = cartesian[1];
				}

				totalDiff = 0;
				double minDist = 0;
				dist = 0;
				// find the distance from every point in closest1 to the closest point in closest2
				for(int p=0;p<closest1.length;p++){
					minDist = Double.POSITIVE_INFINITY;
					for(int q=0;q<closest2.length;q++){
						dist = distance( ((Point)closest1[p].object).x, ((Point)closest2[q].object).x, ((Point)closest1[p].object).y, ((Point)closest2[q].object).y );
						if(dist < minDist){
							minDist = dist;
						}
					}
					totalDiff += minDist;
				}
				// find the distance from every point in closest2 to the closest point in closest1
				for(int p=0;p<closest2.length;p++){
					minDist = Double.POSITIVE_INFINITY;
					for(int q=0;q<closest1.length;q++){
						dist = distance( ((Point)closest2[p].object).x, ((Point)closest1[q].object).x, ((Point)closest2[p].object).y, ((Point)closest1[q].object).y );
						if(dist < minDist){
							minDist = dist;
						}
					}
					totalDiff += minDist;
				}

				if(totalDiff < minDiff){
					minDiff = totalDiff;
				}
			}
		}
		return minDiff;
	}

	// find the key points in the image
	private PixelInfo[] getKeyPointGrid(String imageName,double startRadius,double endRadius,double incRadius,int keyPointCount){
		try{

			img = new RGBImage(imageName);
			int gridSize = 0;

			StringTokenizer st = new StringTokenizer(imageName,".");
			String imagePrefix = st.nextToken();
			String outputImage = imagePrefix + "KeyPoints.jpg";

			String smoothImage = imagePrefix.substring(0,imagePrefix.length()-6);
			RGBImage smoothImg = new RGBImage(smoothImage+".jpg");
			short[][] smoothRed = smoothImg.getRed();
			short[][] smoothGreen = smoothImg.getGreen();
			short[][] smoothBlue = smoothImg.getBlue();

			// Red, green and blue matrices
			short[][] red = img.getRed();
			short[][] green = img.getGreen();
			short[][] blue = img.getBlue();

			int rows = img.getHeight();
			int cols = img.getWidth();

			// convert to a gray scale image
			for(int r=0;r<rows;r++){
				for(int c=0;c<cols;c++){
					red[r][c] = (short)((red[r][c]+green[r][c]+blue[r][c])/3);
					smoothRed[r][c] = (short)((smoothRed[r][c]+smoothGreen[r][c]+smoothBlue[r][c])/3);
				}
			}

			// create an array to store the key points
			PixelInfo[] pixelInfo = new PixelInfo[keyPointCount];
			for(int i=0;i<pixelInfo.length;i++){
				pixelInfo[i] = new PixelInfo(0,0);
			}

			int counter = 0;
			boolean mustSort = true;

			averageArea = new double[rows][cols];

			circleRadius = startRadius;

			for(circleRadius = startRadius;circleRadius<=endRadius;circleRadius*=incRadius){

				areaRadius = circleRadius/2;

				System.err.println("\nRadius: " + circleRadius + "\n");

				double min = Double.POSITIVE_INFINITY;
				double max = Double.NEGATIVE_INFINITY;
				for(int r=0;r<red.length;r++){
					for(int c=0;c<red[r].length;c++){
						averageArea[r][c] = getAreaAverage(red, r, c, areaRadius);
						if(averageArea[r][c] < min){
							min = averageArea[r][c];
						}
						if(averageArea[r][c] > max){
							max = averageArea[r][c];
						}
					}
				}

				gradThresh = (max-min)/8.0;

				gridSize = (int)(2*areaRadius);

				for(int gr=0;gr<red.length;gr+=gridSize){
					System.err.println(gr);
					for(int gc=0;gc<red[gr].length;gc+=gridSize){

						PixelInfo[] pi = getKeyPoints(cols,rows,red, 1, gr, gc, gr+gridSize-1, gc+gridSize-1);

						for(int i=0;i<pi.length;i++){
							if(counter < keyPointCount){
								pixelInfo[counter] = pi[i];
							}else{
								if(mustSort){
									Arrays.sort(pixelInfo);
									mustSort = false;
								}
								if(pi[i].compareTo(pixelInfo[pixelInfo.length-1]) < 0){
									pixelInfo[pixelInfo.length-1] = pi[i];
									mustSort = true;
								}
							}
							counter++;
						}

					}
				}
			}

			img.useRGBColormap();

			for(int i=0;i<pixelInfo.length;i++){

				PixelInfo pixel = pixelInfo[i];

				circleRadius = pixel.radiusValue;
				areaRadius = circleRadius/2;
				double[][] circlePoints = getCirclePoints(pixel.radiusValue);
				pixel.circleValues = getCircleGradients(smoothRed, pixel.row, pixel.col, circlePoints);

				System.err.println(pixel);

				double max = Double.NEGATIVE_INFINITY;
				pixel.maxDirections = new Vector();

				for(int d=0;d<pixel.circleValues.length;d++){
					if(pixel.circleValues[d] > max){
						max = pixel.circleValues[d];
						pixel.maxDirections = new Vector();
						pixel.maxDirections.add(new Integer(d));
					}else if(pixel.circleValues[d]==max){
						pixel.maxDirections.add(new Integer(d));
					}
				}

				for(int r=-1;r<=1;r++){
					for(int c=-1;c<=1;c++){
						if(r!=0 || c!=0){
							red[pixel.row+r][pixel.col+c] = 255;
							green[pixel.row+r][pixel.col+c] = 0;
							blue[pixel.row+r][pixel.col+c] = 0;
						}
					}
				}

			}

			img.saveAsJPEG(outputImage,1);

			return pixelInfo;

		}catch(Exception e){
			e.printStackTrace();
		}
		return null;
	}

	// get the key points in a specific region of the image
	private PixelInfo[] getKeyPoints(int baseImgCols,int baseImgRows,short[][] img,int keyPointCount,int startRow,int startCol,int endRow,int endCol){

		double[][] circlePoints = getCirclePoints(circleRadius);

		PixelInfo[] pixelInfo = new PixelInfo[keyPointCount];
		for(int i=0;i<pixelInfo.length;i++){
			pixelInfo[i] = new PixelInfo(0,0);
		}
		PixelInfo pixel;

		int counter = 0;

		boolean mustSort = true;

		double twoTimesRadius = circleRadius+areaRadius;

		for(int r=startRow;r<=endRow;r++){
			for(int c=startCol;c<=endCol;c++){

				if(r-twoTimesRadius >= 0 && r+twoTimesRadius < img.length && c-twoTimesRadius >= 0 && c+twoTimesRadius < img[0].length){

					double rPercent = (r+1)/(double)img.length;
					double cPercent = (c+1)/(double)img[0].length;
					pixel = new PixelInfo((int)(rPercent*baseImgRows)-1,(int)(cPercent*baseImgCols)-1);
					pixel.radiusValue = circleRadius;

					pixel.pixelValue = img[r][c];

					pixel.circleValues = getCircleGradients(img,r,c,circlePoints);

					int longestChain = 0;
					double longestChainMag = 0;
					int chainStart = 0;
					int chainEnd = 0;
					for(int i=0;i<pixel.circleValues.length;i++){
						int chainLength = 1;
						double chainMag = pixel.circleValues[i];
						int j = (i+1)%pixel.circleValues.length;
						while(chainLength < pixel.circleValues.length && Math.abs(pixel.circleValues[j]) > gradThresh && pixel.circleValues[j]*chainMag > 0){
							chainMag += pixel.circleValues[j];
							chainLength++;
							j = (j+1)%pixel.circleValues.length;
						}
						chainMag = Math.abs(chainMag)/chainLength;
						if(longestChain < chainLength || (longestChain==chainLength && longestChainMag < chainMag)){
							longestChain = chainLength;
							longestChainMag = chainMag;
							chainStart = i;
							chainEnd = (i+(chainLength-1))%pixel.circleValues.length;
						}
						if(i+1 < j){
							i = j-1;
						}
					}
					pixel.longestChain = longestChain;
					pixel.longestChainMag = longestChainMag;
					pixel.chainStart = chainStart;
					pixel.chainEnd = chainEnd;

					if(counter < keyPointCount){
						pixelInfo[counter] = pixel;
					}else{
						if(mustSort){
							Arrays.sort(pixelInfo);
							mustSort = false;
						}
						if(pixel.compareTo(pixelInfo[pixelInfo.length-1]) < 0){
							pixelInfo[pixelInfo.length-1] = pixel;
							mustSort = true;
						}
					}

					counter++;
				}

			}
		}

		return pixelInfo;
	}

	// determine the gradients in the directions at each of the circle points.
	private double[] getCircleGradients(short[][] img, double row, double col, double[][] circlePoints){
		double centerValue = bilinearInterp(averageArea,row,col);
		double[] circleValues = new double[circlePoints.length];
		for(int p=0;p<circleValues.length;p++){
			circleValues[p] = centerValue - bilinearInterp(averageArea,row+circlePoints[p][0],col+circlePoints[p][1]);
		}
		return circleValues;
	}

	// starts out pointing down and rotates counter clockwise
	private double[][] getCirclePoints(double radius){
		double[][] points = new double[pointCount][];
		double stepSize = (2*Math.PI)/(double)pointCount;
		int ndx = 0;
		for(double theta = 0; ndx<points.length && theta<2*Math.PI;theta+=stepSize){
			points[ndx]= convertToCartesian(radius,theta);
			ndx++;
		}
		return points;
	}

	// the the average value over and area of radius "radius" centered at realR and realC
	private double getAreaAverage(short[][] img, double realR, double realC, double radius){
		double sum = 0;
		double totalWeight = 0;
		double distance,weight;
		for(double r=realR-radius; r<=realR+radius; r++ ){

			for(double c=realC-radius; c<=realC+radius; c++ ){

				if(r >= 0 && r < img.length && c >= 0 && c < img[0].length){
					distance = distance(realR,r,realC,c);

					weight = 1;
					if(distance > 0){
						weight = radius-(distance/2.0);
					}
					if( distance <= radius ){
						sum += weight*bilinearInterp(img,r,c);
						totalWeight += weight;
					}
				}
			}
		}
		if(totalWeight > 0){
			return sum/totalWeight;
		}else{
			return -1;
		}
	}

	private double[] convertToPolar(double x,double y){
		if(x==0 && y==0){
			return new double[2];
		}
		double[] points = new double[2];
		points[0] = Math.sqrt(Math.abs(x)+Math.abs(y));
		if(x < 0){
			points[1] = Math.atan(-y/-x)+(Math.PI/2)+Math.PI;
		}else{
			points[1] = Math.atan(y/x)+(Math.PI/2);
		}
		return points;
	}

	private double[] convertToCartesian(double radius,double theta){
		double[] points = new double[2];
		points[0] = radius*Math.cos(theta);
		points[1] = radius*Math.sin(theta);
		return points;
	}

	private double distance(double x1,double x2,double y1,double y2){
		return Math.sqrt( Math.pow(x1-x2,2) + Math.pow(y1-y2,2) );
	}

	private double bilinearInterp(short[][] img, double realR, double realC){

		int rLow = (int)Math.ceil(realR-1);
		int rHigh = (int)Math.ceil(realR);
		int cLow = (int)Math.ceil(realC-1);
		int cHigh = (int)Math.ceil(realC);

		double w1 = distance(realR,rLow,realC,cLow);
		double w2 = distance(realR,rLow,realC,cHigh);
		double w3 = distance(realR,rHigh,realC,cLow);
		double w4 = distance(realR,rHigh,realC,cHigh);

		if(w1 == 0){
			w1 = 1;
			w2 = 0;
			w3 = 0;
			w4 = 0;
		}else if( w2 == 0){
			w1 = 0;
			w2 = 1;
			w3 = 0;
			w4 = 0;
		}else if( w3 == 0){
			w1 = 0;
			w2 = 0;
			w3 = 1;
			w4 = 0;
		}else if( w4 == 0){
			w1 = 0;
			w2 = 0;
			w3 = 0;
			w4 = 1;
		}else{
			double minW = Math.min( Math.min( Math.min(w1,w2) , w3) , w4);
			w1 = minW/w1;
			w2 = minW/w2;
			w3 = minW/w3;
			w4 = minW/w4;
		}

		double total = w1+w2+w3+w4;
		w1 = w1/total;
		w2 = w2/total;
		w3 = w3/total;
		w4 = w4/total;

		double[][] pairs = { 	{rLow,cLow,w1},
				  				{rLow,cHigh,w2},
				  				{rHigh,cLow,w3},
				  				{rHigh,cHigh,w4} };

		double sum = 0;
		int count = 0;
		for(int i=0;i<pairs.length;i++){
			int row = (int)(pairs[i][0]);
			int col = (int)(pairs[i][1]);
			if(row >= 0 && row < img.length && col >= 0 && col < img[0].length){
				count++;
				sum += (pairs[i][2] * img[ row ][ col ]);
			}
		}

		if(count != 0){
		   return sum;
		}else{
		   return -1;
		}
	}

	private double bilinearInterp(double[][] img, double realR, double realC){

		int rLow = (int)Math.ceil(realR-1);
		int rHigh = (int)Math.ceil(realR);
		int cLow = (int)Math.ceil(realC-1);
		int cHigh = (int)Math.ceil(realC);

		double w1 = distance(realR,rLow,realC,cLow);
		double w2 = distance(realR,rLow,realC,cHigh);
		double w3 = distance(realR,rHigh,realC,cLow);
		double w4 = distance(realR,rHigh,realC,cHigh);

		if(w1 == 0){
			w1 = 1;
			w2 = 0;
			w3 = 0;
			w4 = 0;
		}else if( w2 == 0){
			w1 = 0;
			w2 = 1;
			w3 = 0;
			w4 = 0;
		}else if( w3 == 0){
			w1 = 0;
			w2 = 0;
			w3 = 1;
			w4 = 0;
		}else if( w4 == 0){
			w1 = 0;
			w2 = 0;
			w3 = 0;
			w4 = 1;
		}else{
			double minW = Math.min( Math.min( Math.min(w1,w2) , w3) , w4);
			w1 = minW/w1;
			w2 = minW/w2;
			w3 = minW/w3;
			w4 = minW/w4;
		}

		double total = w1+w2+w3+w4;
		w1 = w1/total;
		w2 = w2/total;
		w3 = w3/total;
		w4 = w4/total;

		double[][] pairs = { 	{rLow,cLow,w1},
				  				{rLow,cHigh,w2},
				  				{rHigh,cLow,w3},
				  				{rHigh,cHigh,w4} };

		double sum = 0;
		int count = 0;
		for(int i=0;i<pairs.length;i++){
			int row = (int)(pairs[i][0]);
			int col = (int)(pairs[i][1]);
			if(row >= 0 && row < img.length && col >= 0 && col < img[0].length){
				count++;
				sum += (pairs[i][2] * img[ row ][ col ]);
			}
		}

		if(count != 0){
		   return sum;
		}else{
		   return -1;
		}
	}

	private class Point {

		double x;
		double y;

		public Point(double x,double y){
			this.x = x;
			this.y = y;
		}

	}

	private class PixelInfo implements Comparable {

		private int row;
		private int col;
		private double pixelValue;
		private double[] circleValues;
		private double radiusValue;
		private Vector maxDirections;

		private int longestChain;
		private double longestChainMag;
		private int chainStart;
		private int chainEnd;

		public PixelInfo(int r,int c){
			row = r;
			col = c;
			pixelValue = 0;
			circleValues = new double[pointCount];
		}

		public String toString(){
			return col + "\t" + row + "\t" + radiusValue + "\t" + longestChain + "\t" + longestChainMag;
		}

		public int compareTo(Object z){
			PixelInfo p = (PixelInfo)z;

			if( longestChain > p.longestChain){
				return -1;
			}else if(longestChain < p.longestChain){
				return 1;
			}else{
				if(longestChainMag > p.longestChainMag){
					return -1;
				}else if(longestChainMag < p.longestChainMag){
					return 1;
				}else{
					return 0;
				}
			}
		}

	}

}